@@ -38,7 +38,7 @@ use strict;
 
 my $flag_on_error = 0;
 my $STATUS=0;
-$SIG{"INT"} = sub { $STATUS=2; die $^O eq "MSWin32" ? "die INT\n" : "\n";};
+$SIG{'INT'} = sub { $STATUS=2; die $^O eq 'MSWin32' ? "die INT\n" : "\n";};
 
 END {
   $?=$STATUS if $STATUS;
@@ -56,35 +56,35 @@ END {
 $| = 1;
 foreach my $arg (@ARGV) {
   my ($key,$val) = split /=/, $arg;
-  if ($key eq "--output" or $key eq "-o") {
-    open(OUT, ">", $val);
+  if ($key eq '--output' or $key eq '-o') {
+    open(OUT, '>', $val);
     select OUT;
     $| = 1;
-  } elsif ($key eq "--echo" or $key eq "-e") {
-    print $val, " ";
-  } elsif ($key eq "--ppid" or $key eq "-p") {
+  } elsif ($key eq '--echo' or $key eq '-e') {
+    print $val, ' ';
+  } elsif ($key eq '--ppid' or $key eq '-p') {
     # On MSWin32, getppid() is broken. 
-    my $ppid = $^O eq "MSWin32" ? $ENV{_FORK_PPID} : getppid();
-    print $ppid, " ";
-  } elsif ($key eq "--pid" or $key eq "-P") {
-    my $pid = $^O eq "MSWin32" ? $ENV{_FORK_PID} : $$;
-    print $pid, " ";
-  } elsif ($key eq "--winpid" or $key eq "-W") {
-    print $$, " ";
-  } elsif ($key eq "--sleep" or $key eq "-s") {
+    my $ppid = $^O eq 'MSWin32' ? $ENV{_FORK_PPID} : getppid();
+    print $ppid, ' ';
+  } elsif ($key eq '--pid' or $key eq '-P') {
+    my $pid = $^O eq 'MSWin32' ? $ENV{_FORK_PID} : $$;
+    print $pid, ' ';
+  } elsif ($key eq '--winpid' or $key eq '-W') {
+    print $$, ' ';
+  } elsif ($key eq '--sleep' or $key eq '-s') {
     sleep $val || 1;
-  } elsif ($key eq "--exit" or $key eq "-x") {
+  } elsif ($key eq '--exit' or $key eq '-x') {
     $flag_on_error = 0;
     exit $val || 0;
-  } elsif ($key eq "--input" or $key eq "-y") {
+  } elsif ($key eq '--input' or $key eq '-y') {
     my $y = <STDIN>;
     while ($val-- > 0) {
       print $y;
     }
     print STDERR "received message $y";
-  } elsif ($key eq "--newline" or $key eq "-n") {
+  } elsif ($key eq '--newline' or $key eq '-n') {
     print "\n";
-  } elsif ($key eq "--flag" or $key eq "-F") {
+  } elsif ($key eq '--flag' or $key eq '-F') {
     $flag_on_error = 1;
   }
 }
